#!/usr/bin/perl
#==============================================================================
#
# Name:		slideviewer.cgi
#
# Written:	Peter Sundstrom (Peter.Sundstrom@daedalus.com.au)
#
# Created:	Feb 1997
#
# Source:	http://www.daedalus.com.au/peter/tools/slideviewer/
#
# Description:	Multi-purpose slide viewer.  Specify a file with a list
#		of pictures and descriptions and it does the rest.
#
# Notes:	Image sizing code is adapted and taken from parts of WWWis.
#		http://www.tardis.ed.ac.uk/~ark/wwwis/
#
# Copyright:	(c)1997 Peter Sundstrom.  
#
#		You are free to use and distribute this code as long as the
#		original authors name appears in the header and credits.
#		
#		This code can not be used for commercial gain.
#
# Updated:	30 September 1997
#
# Version:	1.5.1
#
#==============================================================================

# If we are using perl 5 we can use the socket module.  If you have problems
# with this try using the other one.

if ( int($]) >= 5 ){
	eval "use Socket;";
} 
else {
	eval "require 'sys/socket.ph';";
}


#-------------------------------------------------
# CONFIGURABLE OPTIONS
#-------------------------------------------------
 
# URL to this script
$CGI="/peter/cgi-bin/slideviewer.cgi";

# Full path to where your web document root is. 
$WebDocs='/info/www';

# Define URL mappings (aka aliases) that any of the images may use
# First entry is the URL map and the second entry is the pathname it
# expands to.
#
# This is required if you use username maps, for example
#
# '~peters',	'/home/peters/public_html',
#
# Comment this out if you have no mappings
%URLMappings= (
	'/peter',	'/home/peters/www',
);

# Location of proxy server if use one, otherwise comment out.
# This is only required if any of the images are not local.
#$Proxy="http://proxy.daedalus.com.au:8080";

# Full path to an HTML footer file to include things like a naviagtion bar.
$HtmlFooter="/home/peters/www/templates/footer";	

# Full path to the directory where the slide lists are stored
$SlideLists="/home/peters/www/slideshows";

# Extention for the slide lists.  Can be left blank.
$SlideListsExt=".slides";

# URL to the page to display when the show finishes
$End="/peter/end.html";

# URL to images
$Images="/peter/images";

# URL to previous buttons
$PrevIconOff="$Images/back1.gif";
$PrevIconOn="$Images/back2.gif";

# URL to next buttons
$NextIconOff="$Images/next1.gif";	
$NextIconOn="$Images/next2.gif";	

# URL to start buttons
$StartIconOff="$Images/top1.gif";
$StartIconOn="$Images/top2.gif";

# URL to end button
$EndIconOff="$Images/bottom1.gif";
$EndIconOn="$Images/bottom2.gif";

# Icon Sizes
$IconHeight=40;
$IconWidth=50;

# Default refresh rate in seconds
$DefRefresh=20;

# The amount to adjust the refresh rate when going faster or slower (in secs)
$Adjust=5;

# The amount to reduce or enlarge an image
$ScaleFactor=1.25;

# Default direction for the show.  Can be 'forward' or 'backward'
$Direction='forward';	

# If you want to change background colours, heading etc, change
# the HtmlPage subroutine.

#-----------------------------------------------------------------
# END OF CONFIGURABLE OPTIONS
#=================================================================

$Query = $ENV{QUERY_STRING};
%Data = ();

&Error("No data") if (! "$Query");

foreach(split('&', $ENV{QUERY_STRING})) {
  ($name, $value) = split('=', $_);
  $value =~ s!\+! !g; $value =~ s!%(..)!pack('c',hex($1))!eg;
  $Data{$name} = $value;                
}


#
# Open the specified slide list
#
$ListName=$Data{list};
$List="$SlideLists/$Data{list}$SlideListsExt";

&Error("No slide show list specified") unless "$ListName";
open (LIST,"$List") || &Error("Can not open slide show $List");

#
# Set the recyle option
#
if ($Data{cycle} eq 'on') {
	$Cycle=$Data{cycle};
}
else {
	$Cycle='off';
}

# 
# Determine direction to go in
#
$Direction=$Data{direction} if ($Data{direction});

if ($Direction eq 'forward') {
	$OtherWay='backward';
}
elsif ($Direction eq 'backward') {
	$OtherWay='forward';
}
else {
	$Direction='forward';
	$OtherWay='backward';
}

#
# Determine what slide number we are up to
#
if ($Data{slide} && $Data{slide} > 1) {
	$Slide=$Data{slide};
	if ($Direction eq 'forward') {
		$Prev=$Slide - 1;
		$Next=$Slide + 1;
	}
	else {
		$Prev=$Slide + 1;
		$Next=$Slide - 1;
	}
}
else {
	$Slide=1;
	$Prev=1;
	$Next=2;
}


#
# Extract the image URL, description and Subtitle for the appropriate 
# slide number.
#
$Count=1;

while (<LIST>) {
	next if (/^#/ or /^$/);

	if (/^:include/) {
		s/:include //;
		$IncList=$_;
		open (INCLIST,"$SlideLists/$IncList") || &Error("Can not open included slide show $IncList");
		while (<INCLIST>) {
			next if (/^#/ or /^$/);
 			if ($Count == $Slide) {
				chomp; 
				($Image,$Description,$Subtitle)=split(/\|/);
			}
			$Count++;
		}
		close (INCLIST);
	}
	
 	if ($Count == $Slide) {
		chomp; 
		($Image,$Description,$Subtitle)=split(/\|/);
	}

	$Count++;
}

close (LIST);

$Total=$Count - 1;

&Error("Slide number out of range") if ($Slide > $Total);

#
# Set the refresh rate if specified
#
$Refresh=$Data{refresh} if ($Data{refresh} > 0);

#
# Set refresh times
#
$Slower=$Refresh + $Adjust;
$Faster = $Refresh - $Adjust if ($Refresh > $Adjust);

#
# Determine HEIGHT and WIDTH tags from image if not specified
#
if ($Data{width}) {
	$Width=$Data{width};
	$Height=$Data{height};
}
elsif ($Data{scale}) {
	$ImagePath=&ImageLocation("$Image");
	($Width,$Height)=&imgsize("$ImagePath");
	$Scale=$Data{scale};

	if ($Scale < 0) {
		foreach ($i=0; $i>$Scale; $i--) {
			$Width = $Width / $ScaleFactor;
			$Height = $Height / $ScaleFactor;
		}
	}
	else {
		foreach ($i=0; $i < $Scale; $i++) {
			$Width = $Width * $ScaleFactor;
			$Height = $Height * $ScaleFactor;
		}
	}
}
else {
	$ImagePath=&ImageLocation("$Image");
	($Width,$Height)=&imgsize($ImagePath);
}


#
# Calculate enlargement or reduction if selected.
#
if ($Data{scale}) {
	$Scale=$Data{scale};
}
else {
	$Scale=0;
}

if ($Data{size} eq 'enlarge') {
	$Scale++;
	$Width = $Width * $ScaleFactor;
	$Height = $Height * $ScaleFactor; 
}
elsif ($Data{size} eq 'reduce') {
	$Scale--;
	$Width = $Width / $ScaleFactor;
	$Height = $Height / $ScaleFactor; 
}

#
# If this is the last slide set the end page or go back to the
# beginning if cycle mode is on.
#
if ($Next > $Total or ($Slide == 1 && $Direction eq 'backward')) {
	if ($Cycle eq 'on') {
		$URL="$CGI?list=${ListName}&refresh=$Refresh&direction=$Direction&cycle=on&scale=$Scale" if ($Direction eq 'forward');
		$URL="$CGI?list=${ListName}&refresh=$Refresh&direction=$Direction&scale=$Scale&cycle=on&slide=$Total" if ($Direction eq 'backward');
	}
	else {
		$URL="$End";
	}
}
else {
	$URL="$CGI?list=${ListName}&refresh=$Refresh&direction=$Direction&scale=$Scale&cycle=$Cycle&slide=$Next";
}	

&HtmlPage;
&ReadFooter;

#-----------------------------------------------------------------------------
sub HtmlPage {

	print "Content-type: text/html\n\n";
 	print "<html>\n<head>\n"; 

	if ($Refresh > 0) {
		print "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"$Refresh;URL=$URL\">\n";
	}

	&JavaScript;

	print <<HTML;
<title>$Description</title>
</head>
<body bgcolor="#FFFFFF">

<center>
<font color="#00885D" face="comic Sans MS, Arial" size=6>
$Description<br>
</font>
HTML

	# 
	# Display a subtitle (if one exists)
	#
	print "<font color=#00885D face=\"comic Sans MS, Arial\" size=3>$Subtitle</font><br>\n" if ($Subtitle);

	print <<HTML;
<p>

<table border=5>
<tr>
<td align=center>
HTML

	#
	# If height and width sizes have been calculated, use them
	#
	if ($Height > 0 ) {
		print "<a href=\"$URL\"><img src=$Image height=$Height width=$Width border=0 alt=\"$Description\"></a>\n";
	}
	else {
		print "<a href=\"$URL\"><img src=$Image border=0 alt=\"$Description\"></a>\n";
	}
	
	print <<HTML
</td>
</tr>
</table>
<br>

<form action=$CGI>
<input type=hidden name=list value=$ListName>
<input type=hidden name=refresh value=$Refresh>
<input type=hidden name=cycle value=$Cycle>
<input type=hidden name=scale value=$Scale>
<input type=hidden name=direction value=$Direction>
Slide <input type=text name=slide value=$Slide size=3> of $Total
HTML
;
	if ($Refresh > 0 ) {
 		print "(Refresh $Refresh secs)<p>\n";
	}
	else {
		print "(Refresh off)<p>\n";
	}

	print <<HTML
</form>

<a href="$CGI?list=$ListName&refresh=$Refresh&slide=$Prev&cycle=$Cycle&scale=$Scale" onMouseOver="img_on('back','Go back to the previous slide');return true" onMouseOut="img_off('back')"><img src="$PrevIconOff" name="back" border=0 height=$IconHeight width=$IconWidth alt="Previous"></a>
<a href="$CGI?list=$ListName&refresh=$Refresh&cycle=$Cycle&scale=$Scale" onMouseOver="img_on('top','Go to the start of the slide show');return true" onMouseOut="img_off('top')"><img src="$StartIconOff" name="top" border=0 height=$IconHeight width=$IconWidth alt="Start"></a>
<a href="$End" onMouseOver="img_on('bottom','Go to the end of the slide show'); return true" onMouseOut="img_off('bottom')"><img src="$EndIconOff" name="bottom" border=0 height=$IconHeight width=$IconWidth alt="End"></a>
<a href="$URL" onMouseOver="img_on('next','Go to the next slide'); return true" onMouseOut="img_off('next')"><img src="$NextIconOff" name="next" border=0 height=$IconHeight width=$IconWidth alt="Next"></a> <p>

<a href="$CGI?list=$ListName&slide=$Prev&refresh=$Refresh&cycle=$Cycle&scale=$Scale" onMouseOver="self.status='Go back to the previous slide';return true" onMouseOut="self.status=''">Previous</a> |
<a href="$URL" onMouseOver="self.status='Go to the next slide';return true" onMouseOut="self.status=''">Next</a> |
<a href="$CGI?list=$ListName&refresh=$Refresh&scale=$Scale" onMouseOver="self.status='Go back to the beginning of the slide show';return true" onMouseOut="self.status=''">Start</a> |
<a href="$CGI?list=$ListName&slide=$Total&direction=$Direction&refresh=$Refresh&scale=$Scale&cycle=$Cycle" onMouseOver="self.status='Go to the end of the slide show';return true" onMouseOut="self.status=''">Last</a> |
<a href="$End" onMouseOver="self.status='Finish the slide show';return true" onMouseOut="self.status=''">End</a>
HTML
;
	if ($Refresh > 0) {
		print "| <a href=\"$CGI?list=${ListName}&refresh=$Faster&direction=$Direction&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Speed up the refreshes\';return true\" onMouseOut=\"self.status=\'\'\">Faster</a> |\n";
		print "<a href=\"$CGI?list=${ListName}&refresh=$Slower&direction=$Direction&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Slow down the refreshes\';return true\" onMouseOut=\"self.status=\'\'\">Slower</a> \n";
		print "<br> <a href=\"$CGI?list=$ListName&refresh=-1&direction=$Direction&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Turn auto-refreshes off\';return true\" onMouseOut=\"self.status=\'\'\">Auto-refresh Off</a>\n";
	}
	else {
		print "<br><a href=\"$CGI?list=$ListName&refresh=$DefRefresh&direction=$Direction&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Turn auto-refreshes on\';return true\" onMouseOut=\"self.status=\'\'\">Auto-refresh On</a>\n";
	}

	print "| <a href=\"$CGI?list=$ListName&refresh=$Refresh&size=enlarge&width=$Width&height=$Height&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Enlarge the current image\';return true\" onMouseOut=\"self.status=\'\'\">Enlarge</a> ";
	print "| <a href=\"$CGI?list=$ListName&refresh=$Refresh&scale=0&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Display the current image at its normal size\';return true\" onMouseOut=\"self.status=\'\'\">Normal Size</a> ";
	print "| <a href=\"$CGI?list=$ListName&refresh=$Refresh&size=reduce&width=$Width&height=$Height&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Reduce the size of the current image\';return true\" onMouseOut=\"self.status=\'\'\">Reduce</a> ";
	print "| <a href=\"$CGI?list=${ListName}&refresh=$Refresh&direction=$OtherWay&scale=$Scale&cycle=$Cycle&slide=$Slide\" onMouseOver=\"self.status=\'Reverse the direction of the slide show\';return true\" onMouseOut=\"self.status=\'\'\">Reverse direction</a><p>\n";


}
 
#-----------------------------------------------------------------------------
sub JavaScript {

	print <<HTML;
<script language="JavaScript">

<!--
browserName = navigator.appName;
browserVer = parseInt(navigator.appVersion);
if (browserName == "Netscape" && browserVer >= 3) version = "n3";
else version = "braindead"; 

if (version == "n3")
{       
	backon = new Image; backon.src="$PrevIconOn";
	backoff = new Image ($IconHeight,$IconWidth); backoff.src="$PrevIconOff";
	topon = new Image; topon.src="$StartIconOn";
	topoff = new Image ($IconHeight,$IconWidth); topoff.src="$StartIconOff";
	nexton = new Image; nexton.src="$NextIconOn";
	nextoff = new Image ($IconHeight, $IconWidth); nextoff.src="$NextIconOff";
	bottomon = new Image; bottomon.src="$EndIconOn";
	bottomoff = new Image ($IconHeight,$IconWidth); bottomoff.src="$EndIconOff";
}

function img_on(imgName,Text)
{
	self.status=Text;

	if (version == "n3") {
		imgOn = eval (imgName + "on.src");
		document [imgName].src = imgOn;         
	}
}

function img_off (imgName)
{ 
	self.status='';

	if (version == "n3") {           
		imgOff = eval (imgName + "off.src");
		document [imgName].src = imgOff;        
	}
}

// -->

</script>
HTML
}

#-----------------------------------------------------------------------------
sub ImageLocation {
	local ($Image) = @_;

	return("$WebDocs/$Image") unless %URLMappings;
	return("$Image") if ($Image =~ /http/);

	while (($Key,$Value) = each %URLMappings) {
		$Key =~ s%/%\\/%g;
		if ($Image =~ /$Key/) {
			($ImagePath=$Image) =~ s%$Key%$Value%;
			last;
		}
	}

	if ($ImagePath) {
		return("$ImagePath");
	}
	else {
		return("$WebDocs/$Image");
	}
}

#-----------------------------------------------------------------------------
sub ReadFooter {
	open(FOOTER,"$HtmlFooter") || &Error1("Can not open footer: $HtmlFooter");
	
	while (<FOOTER>) {
		print;
	}

	close(FOOTER);
}

#-----------------------------------------------------------------------------
sub Error {
	local($Text)=@_;

	print "Content-type: text/html\n\n";
	print <<HTML;
<html>
<head>
<title>Slide Show Error</title>
<body bgcolor="#ffffff">
<center>
<h1>Slide Show Error</h1>
</center>
<p>
$Text<p>

HTML

	&ReadFooter;
	exit;
}

#-----------------------------------------------------------------------------
sub Error1 {
	local($Text)=@_;

	print "Content-type: text/html\n\n";
	print <<HTML;
<html>
<head>
<title>Slide Show Error</title>
<body bgcolor="#ffffff">
<center>
<h1>Slide Show Error</h1>
</center>
<p>
$Text<p>
</body>
</html>

HTML

	exit;
}
#-----------------------------------------------------------------------------
# Code adapted from wwwis
#
# looking at the filename really sucks I should be using the first 4 bytes
# of the image. If I ever do it these are the numbers.... (from chris@w3.org)
#  PNG 89 50 4e 47    
#  GIF 47 49 46 38
#  JPG ff d8 ff e0
#  XBM 23 64 65 66

sub imgsize {
	local($file)= shift @_;
	local($x,$y)=(0,0);
  
	if ($file =~ /http/i) {
		($x,$y) = &URLsize($file);
	}
	elsif ( defined($file) && open(STREAM, "<$file") ) {
    		binmode( STREAM ); # for crappy MS OSes - Win/Dos/NT use is NOT SUPPORTED
	if ($file =~ /\.jpg$/i || $file =~ /\.jpeg$/i) {
		($x,$y) = &jpegsize(STREAM);
    	} 
	elsif ($file =~ /\.gif$/i) {
		($x,$y) = &gifsize(STREAM);
	} 
	elsif($file =~ /\.xbm$/i) {
		($x,$y) = &xbmsize(STREAM);
	} 
	elsif($file =~ /\.png$/i) {
		($x,$y) = &pngsize(STREAM);
	} 
	else {
		&Error("$file is not gif, xbm, jpeg or png");
	}
	close(STREAM);

	}
      
	return ($x,$y);
}


#-----------------------------------------------------------------------------
sub gifsize
{
	local($GIF) = @_;
	local($type,$a,$b,$c,$d,$s)=(0,0,0,0,0,0);

	if (defined($GIF) && read($GIF, $type, 6) &&
	    $type =~ /GIF8[7,9]a/ && read($GIF, $s, 4) == 4 ) {
		($a,$b,$c,$d)=unpack("C"x4,$s);
		return ($b<<8|$a,$d<<8|$c);
	}

	return (0,0);
}

#-----------------------------------------------------------------------------
sub xbmsize {
	local($XBM) = @_;
	local($input)="";
  
	if (defined($XBM)) {
		$input .= <$XBM>;
		$input .= <$XBM>;
		$input .= <$XBM>;
		$_ = $input;

		if (/.define\s+\S+\s+(\d+)\s*\n.define\s+\S+\s+(\d+)\s*\n/i) {
			return ($1,$2);
    		}
	}

	return (0,0);
}

#-----------------------------------------------------------------------------
#  pngsize : gets the width & height (in pixels) of a png file
# cor this program is on the cutting edge of technology! (pity it's blunt!)
#  GRR 970619:  fixed bytesex assumption

sub pngsize {
	local($PNG) = @_;
	local($head) = "";
	local($a, $b, $c, $d, $e, $f, $g, $h)=0;

	if (defined($PNG)				&& 
	   read( $PNG, $head, 8 ) == 8			&&
	   $head eq "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a"  &&
	   read($PNG, $head, 4) == 4			&&
	   read($PNG, $head, 4) == 4			&&
   	   $head eq "IHDR"				&&
           read($PNG, $head, 8) == 8 			) {
		($a,$b,$c,$d,$e,$f,$g,$h)=unpack("C"x8,$head);
		return ($a<<24|$b<<16|$c<<8|$d, $e<<24|$f<<16|$g<<8|$h);
	}

	return (0,0);
}

#-----------------------------------------------------------------------------
# jpegsize : gets the width and height (in pixels) of a jpeg file
# Andrew Tong, werdna@ugcs.caltech.edu           February 14, 1995
# modified slightly by alex@ed.ac.uk

sub jpegsize {
	local($JPEG) = @_;
	local($done)=0;
	local($c1,$c2,$ch,$s,$length, $dummy)=(0,0,0,0,0,0);

	if (defined($JPEG)		&&
	    read($JPEG, $c1, 1)		&&
	    read($JPEG, $c2, 1)		&&
	    ord($c1) == 0xFF		&& 
	    ord($c2) == 0xD8		) {
		while (ord($ch) != 0xDA && !$done) {
			# Find next marker (JPEG markers begin with 0xFF)
      			# This can hang the program!!
      			while (ord($ch) != 0xFF) {  read($JPEG, $ch, 1); }
      			# JPEG markers can be padded with unlimited 0xFF's
      			while (ord($ch) == 0xFF) { read($JPEG, $ch, 1); }
      			# Now, $ch contains the value of the marker.

      			if ((ord($ch) >= 0xC0) && (ord($ch) <= 0xC3)) {
				read ($JPEG, $dummy, 3); read($JPEG, $s, 4);
				($a,$b,$c,$d)=unpack("C"x4,$s);
				return ($c<<8|$d, $a<<8|$b );
      			} else {
				# NOT valid JPEG markers
				read ($JPEG, $s, 2); 
				($c1, $c2) = unpack("C"x2,$s); 
				$length = $c1<<8|$c2;
				last if (!defined($length) || $length < 2);
				read($JPEG, $dummy, $length-2);
      			}
    		}
  	}

  	return (0,0);
}

#-----------------------------------------------------------------------------
sub URLsize {
	local($five) = @_;
	local($dummy, $dummy, $server, $url);
  
	local( $x,$y) = (0,0);
  
	if ($Proxy =~ /\S+/) {
		($dummy, $dummy, $server, $url) = split(/\//, $Proxy, 4);
		$url=$five;
	} 
	else {
		($dummy, $dummy, $server, $url) = split(/\//, $five, 4);
		$url= '/' . $url;
	}
  
	local($them,$port) = split(/:/, $server);

	$port = 80 unless $port;
	$them = 'localhost' unless $them;
  
	$_=$url;

	if (/gif/i || /jpeg/i || /jpg/i || /xbm/i) {
    
		$sockaddr = 'S n a4 x8';
    
		chop ($hostname = `uname -n`);
    
		($name,$aliases,$proto) = getprotobyname('tcp');
		($name,$aliases,$port) = getservbyname($port,'tcp')
		unless $port =~ /^\d+$/;;
		($name,$aliases,$type,$len,$thisaddr) = gethostbyname($hostname);
		($name,$aliases,$type,$len,$thataddr) = gethostbyname($them);
    
		&Error("Unknown server/proxy port")  if (!defined($port));
		&Error("Server address error")    if (!defined($thataddr));
    
		$this = pack($sockaddr, &AF_INET, 0, $thisaddr);
		$that = pack($sockaddr, &AF_INET, $port, $thataddr);

		# Make the socket filehandle.

		if (!((socket(S, &AF_INET, &SOCK_STREAM, $proto)) &&

	   	# Give the socket an address.
	   	(bind(S, $this)) &&

	   	# Call up the server.
	   	(connect(S,$that)) )) {
			# there was a problem
			&Error(" $!");
		} 
		else {
			# Set socket to be command buffered.
			select(S); $| = 1; select(STDOUT);
     	 
			print S "GET $url\n\n";
     	 
			if ($url =~ /\.jpg$/i || $url =~ /\.jpeg$/i) {
				($x,$y) = &jpegsize(S);
			} 
			elsif ($url =~ /\.gif$/i) {
				($x,$y) = &gifsize(S);
      			} 
			elsif ($url =~ /\.xbm$/i) {
        			($x,$y) = &xbmsize(S);
      			} 
			elsif ($url =~ /\.png$/i) {
        			($x,$y) = &pngsize(S);
      			} 
			else {
        			&Error("$url is not gif, jpeg, xbm or png");
			}         
		}
	} 
	else {
		&Error("$url is not gif, xbm or jpeg");
  	}

	return ($x,$y);
}
